package nl.uvt.locator;

import java.util.LinkedList;
import java.util.List;

import nu.xom.*;

public class MerescoInfo extends LinkInfo {
	String epn;
	String status;
	String callNumber;
	String material;
	String collection;
	String action;
	List<Threshold> thresholds; 

	MerescoInfo(){
		epn = "";
		status = "";
		callNumber =  "";
		material = "";
		action = "";
		collection = "";
		thresholds = new LinkedList<Threshold>();
	}

	static XPathContext modsContext = Locator.sruContext;

	public static final String modsNamespace = "http://www.loc.gov/mods/v3";
	public static final String schemaNamespace = "http://www.w3.org/2001/XMLSchema-instance";
	public static final String extNamespace = "http://drcwww.uvt.nl/~place/search%20engine/extension";

	public static void fillMultipleRecord(Node inputRecord, List<LinkInfo> meResult) {
		Locator.logger.debug( "split one record per occurrence");
//		Locator.logger.debug( "input record: " + inputRecord.toXML() );
		MerescoInfo data = new MerescoInfo();
		data.linkMethod = "meresco";
		Nodes nodes= inputRecord.query("./mods:recordInfo/mods:recordIdentifier", MerescoInfo.modsContext );
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				data.linkIssn = nodes.get(0).getChild(0).getValue().toString();
		}
		nodes = inputRecord.query("./mods:titleInfo/mods:title", MerescoInfo.modsContext);
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				data.linkTitle = nodes.get(0).getChild(0).getValue().toString();
		}
		nodes = inputRecord.query("./mods:originInfo/mods:publisher", MerescoInfo.modsContext );
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				data.linkPublisher = nodes.get(0).getChild(0).getValue().toString();
		}
		nodes = inputRecord.query("./mods:location", MerescoInfo.modsContext )	;
		if ( nodes.size() > 0 ){
//			Locator.logger.debug("found " + nodes.size() + " location nodes" );
			if ( nodes.get(0).getChildCount() > 0 ) {
				Nodes copyNodes = nodes.get(0).query("./mods:holdingSimple/mods:copyInformation", MerescoInfo.modsContext);
				int occ = copyNodes.size();
				if ( occ > 0 ){
//					Locator.logger.debug( "found " + occ + " copyInformation nodes");
					String [] occArray = new String[occ];
					for ( int i=0;  i < occ; ++i ){
						Nodes occNotes = copyNodes.get(i).query("./mods:note[@type='occurrence']", MerescoInfo.modsContext );
						if ( occNotes.size() >0 ){
							String occS = occNotes.get(0).getChild(0).getValue().toString();
							occArray[i] = occS;
						}
						else
							Locator.logger.debug("occurence lookup failed for: " + i );
					}
					for ( int i=0;  i < occ; ++i ){
						MerescoInfo rec = new MerescoInfo();
						rec.action = "";
						rec.linkIssn = data.linkIssn;
						rec.linkTitle = data.linkTitle;
						rec.linkPublisher = data.linkPublisher;
						Nodes copies = nodes.get(0).query("./mods:holdingExternal/a:copies", MerescoInfo.modsContext);
						if ( copies.size() == 0 ){
							// use what we got in simple
							Locator.logger.debug( "found no external holding nodes");
							Node holding = null;
							for( int j=0; j < occ && holding == null; ++j ){
								Node tmp = copyNodes.get(j).query("./mods:note[@type='occurrence']", MerescoInfo.modsContext).get(0);
								if ( tmp.getChild(0).getValue().toString().equals(occArray[i]) )
									holding = copyNodes.get(j);
							}
							if ( holding != null ) {
//								Locator.logger.debug( "holding node" + holding.toXML() );
								Nodes notes = holding.query("./mods:note[@type='epn']", MerescoInfo.modsContext);
								if( notes.size() > 0 ){
									String epn = notes.get(0).getValue().toString();
									rec.epn = epn;
									Locator.logger.debug("got epn:" + epn );
								}
								notes = holding.query("./mods:note[@type='status']", MerescoInfo.modsContext);
								if( notes.size() > 0 ){
									String status = notes.get(0).getValue().toString();
									rec.status = status;
									Locator.logger.debug("got status: " + status );
								}
								notes = holding.query("./a:shelfLocator", MerescoInfo.modsContext );
								if( notes.size() > 0 ){
									String call = notes.get(0).getValue().toString();
									Locator.logger.debug("got call: " + call );
									fixCall( call, rec );
								}
							}
						}
						else {
							Nodes holdings = copies.get(0).query("./a:copy[a:occurrence='"+ occArray[i] +"']", MerescoInfo.modsContext);
							if ( holdings.size() == 0 ){
								Locator.logger.debug( "found no holding for occ=" + occArray[i] );
							}
							else {
								Node holding = holdings.get(0);
								Locator.logger.debug( "found holding for occ=" + occArray[i] );
								Locator.logger.debug( holding.toXML());
								Nodes info = holding.query("./a:epn", MerescoInfo.modsContext );
								if ( info.size() > 0 ){
									if ( info.get(0).getChildCount() > 0 ) 
										rec.epn = info.get(0).getChild(0).getValue().toString();
								}
								info = holding.query("./a:collection", MerescoInfo.modsContext);
								if ( info.size() > 0 ){
									if ( info.get(0).getChildCount() > 0 ) 
										rec.collection = info.get(0).getChild(0).getValue().toString();
								}
								info = holding.query("./a:status", MerescoInfo.modsContext);
								if ( info.size() > 0 ){
									if ( info.get(0).getChildCount() > 0 ) 
										rec.status = info.get(0).getChild(0).getValue().toString();
								}
								info = holding.query("./a:shelfLocator", MerescoInfo.modsContext);
								if ( info.size() > 0 ){
									if ( info.get(0).getChildCount() > 0 ) 
										rec.callNumber = info.get(0).getChild(0).getValue().toString();
								}
								info = holding.query( "./a:range", MerescoInfo.modsContext);
								for ( int j=0; j < info.size(); ++j ){
									Locator.logger.debug("get range:"+j);
									Threshold th = new Threshold();
									Nodes thNodes = info.get(j).query( "./a:threshold", MerescoInfo.modsContext);
									for ( int k=0; k < thNodes.size(); ++k ){
										Threshold thSub = new Threshold();
										thSub.fill( thNodes.get(k), "a", MerescoInfo.modsContext );
										th.more.add( thSub );
									}
									rec.thresholds.add( th );
								}
							}
						}
						meResult.add( rec );
					}
				}
			}
		}
	}

	private static void fixCall( String call, MerescoInfo rec){
		Locator.logger.debug("fixup " + call );
		if ( !call.equals("") )
			rec.collection = tryToFixCollection( call );
		if ( rec.collection.equals("") )
			rec.callNumber = call;
		else {
			// for now, we leave the collection in the call, to stay backward compatible
//			call = call.replace( rec.collection + " ", "" );
			rec.callNumber = call;
		}
		Locator.logger.debug( "rec.collection="+rec.collection );
		Locator.logger.debug( "rec.callNumber="+ rec.callNumber );
	}	

	private static String tryToFixCollection(String call ) {
		//
		// try to guess the collection from the callnumber
		// rather naive
		//
		String[] knownCollections = {"CBM", "BRA", "DISC", "ECO", "JUR", "LET", "SOC", "TFL" };
		for ( int i=0; i < knownCollections.length; ++i ){
			String tst = knownCollections[i];
			if ( call.contains( tst ))
				return tst;
		}
		return "";	
	}

	void fill( Node rec ){
		logger.debug( "fill from record:" + rec.toXML() );
		linkMethod = "meresco";
		Nodes nodes= rec.query("./mods:recordInfo/mods:recordIdentifier", modsContext);
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				linkIssn = nodes.get(0).getChild(0).getValue().toString();
		}
		nodes = rec.query("./mods:titleInfo/mods:title", modsContext);
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				linkTitle = nodes.get(0).getChild(0).getValue().toString();
		}
		nodes = rec.query("./mods:location/mods:url", modsContext);
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				linkUrl = nodes.get(0).getChild(0).getValue().toString();
		}
		nodes = rec.query("./mods:originInfo/mods:publisher", modsContext);
		if ( nodes.size() > 0 ){
			if ( nodes.get(0).getChildCount() > 0 ) 
				linkPublisher = nodes.get(0).getChild(0).getValue().toString();
		}
	}		

	Node toRecordNode(){
		Element record = new Element( "record" );
		record.appendChild( makeNode( "linkIssn", linkIssn ) );
		record.appendChild( makeNode( "linkTitle", linkTitle ) );
		record.appendChild( makeNode( "linkUrl", linkUrl ) );
		record.appendChild( makeNode( "generatedUrl", generatedUrl ) );
		record.appendChild( makeNode( "linkMethod", linkMethod ) );
		record.appendChild( makeNode( "action", action ) );
		record.appendChild( makeNode( "collection", collection ) );
		record.appendChild( makeNode( "epn", epn ) );
		record.appendChild( makeNode( "callNumber", callNumber ) );
		record.appendChild( makeNode( "material", material ) );
		record.appendChild( makeNode( "status", status ) );
		return record;
	}

	public boolean inRange( QueryData pQ) {
		try {
			return Threshold.inRange( thresholds, pQ );
		} catch (LocatorException e) {
			logger.debug("too bad");
			return false;
		}
	}

	public boolean applyMethod( QueryData pQ ){
		logger.debug("handle Meresco method: " + status );
		logger.debug("action="+action);
		logger.debug("status="+status);
		if ( !inRange( pQ ) ){
			logger.debug( "rejected because of threshold");
			return false;
		}
		else {
//			logger.debug( "collection=" + collection );
//			logger.debug( "epn=" + epn );
//			logger.debug( "status=" + status );
//			logger.debug( "callNumber=" + callNumber );
			if ( action.equals("") || action.equals("passed")){
				if ( !linkUrl.equals("") ){
					action = "url";
					generatedUrl = linkUrl;				
				}
				else if ( !epn.equals("") && !status.equals("") && 
						!callNumber.equals("") && !collection.equals("")) {
					if ( collection.equals("CBM") )
						action = "depot";
					else if ( status.equals("u") )
						action = "reserve";
					else if ( status.equals("f"))
						action = "shelf";
					else if ( status.equals("b"))
						action = "raadpleegbaar";
				}
				else
					action = "reject";
			}
			return !action.equals("reject");
		}
	}

}
